#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "analysisd/eventinfo.h"
#include "os_regex/os_regex.h"

/* Special decoder for snort
 * We are not using the default rendering
 * to make it simple (and less resource intensive)
 */
int DecodeSnort(Eventinfo *lf)
{
    char **ret;
    
    #ifdef DEBUG
    printf("%s: DEBUG: Checking snort decoder.\n",ARGV0);
    #endif
    
    /* Begging of the snort msg */
    if(!OS_Match("^[**] [",lf->log))
        return(0);


    /* Some examples
       [**] [1:1054:7] WEB-MISC weblogic/tomcat .jsp view source attempt [**] 
       [Classification: Web Application Attack] 
       [Priority: 1]  10.4.12.26:34041 -> 66.179.53.37:80

       [**] [1:1421:11] SNMP AgentX/tcp request [**] 
       [Classification: Attempted Information Leak] [Priority: 2]  
       10.4.3.20:626 -> 10.4.10.161:705
     */

    /* getting snort ID */
    lf->type = SNORT;
   
    ret = OS_RegexStr("^[**] [(\\d+:\\d+:\\d+)]\\.+[Priority: \\d+]  "
             "(\\S+):\\d+ ->", lf->log);          

    /* Didn't match */
    if(ret == NULL)
        return(0);
    
    if(ret[0])
    {
        lf->id = strdup(ret[0]);
        free(ret[0]);
    }
    
    if(ret[1])
    {
        lf->srcip = strdup(ret[1]);
        free(ret[1]);
    }
    
    free(ret); 

    /* Snort FTS */
    lf->fts = strdup("name,id,srcip");
    lf->comment = strdup("First time Snort rule fired");
    
    return(1);
}
